from ngsolve import *
from netgen.csg import *
from math import sqrt
from time import time

geo=CSGeometry()

cyl1 = Cylinder(Pnt(0,0,-1),Pnt(0,0,2),0.3).bc("scatterer")
cyl2 = Cylinder(Pnt(0,0,-1),Pnt(0,0,2),0.4).bc("scatterer")
cyl3 = Cylinder(Pnt(0,0,-1),Pnt(0,0,2),0.7)
cyl4 = Cylinder(Pnt(0,0,-1),Pnt(0,0,2),1)

box1 = OrthoBrick(Pnt(-1,-1,0),Pnt(1,1,1)).bc("scatterer")
box2 = OrthoBrick(Pnt(-1,-1,-0.2),Pnt(1,1,1.2)) 
box3 = OrthoBrick(Pnt(-1,-1,-0.4),Pnt(1,1,1.4)) 

pipe=((cyl2-cyl1)*box1).mat("pipe")
air = (cyl3*box2-pipe).mat("air")
pmldom = (cyl4*box3-(cyl3*box2)).mat("pml")

#geo.Add(pipe)
geo.Add(air)
geo.Add(pmldom)

#
ngmesh = geo.GenerateMesh(maxh=0.3)

## ngmesh.Save("scattering.vol")
mesh = Mesh(ngmesh)
mesh.Curve(5)
Draw(mesh)
## mesh = Mesh ("scattering.vol")

p=pml.Compound(pml.Radial(origin=(0,0),rad=0.7),pml.Cartesian((0),(1)))
mesh.SetPML(p,definedon="pml")

kx = 20
ky = 5
kz = 5
k = sqrt(kx*kx+ky*ky+kz*kz)

uin = exp (1J*kx*x+1J*ky*y+1J*kz*z)

fes = H1(mesh, complex=True, order=5, dirichlet="scatterer")
u = fes.TrialFunction()
v = fes.TestFunction()

uscat = GridFunction (fes)
uscat.Set (uin, definedon=mesh.Boundaries("scatterer"))

a = BilinearForm (fes, symmetric=True)
a += SymbolicBFI (grad(u)*grad(v)-k*k*u*v )

f = LinearForm (fes)

start = time()
a.Assemble()
print("Assembling needed ", time()-start, " seconds")
f.Assemble()

res = uscat.vec.CreateVector()
res.data = f.vec - a.mat * uscat.vec
uscat.vec.data += a.mat.Inverse(freedofs=fes.FreeDofs(), inverse="sparsecholesky") * res


Draw (uin, mesh, "uin")
Draw (uscat, mesh, "uscat")
mesh.UnSetPML('pml')
cf=p.PML_CF
Draw (cf-Conj(cf),mesh,"scaling")
pml_indicator = CoefficientFunction([0 if mat=="pml" else 1 for mat in mesh.GetMaterials()])
Draw (pml_indicator * (uin-uscat), mesh, "utot")

#visualization
from ngsolve.internal import *


viewoptions.clipping.enable = 1
viewoptions.clipping.ny = 1
viewoptions.clipping.nz = 0
visoptions.clipsolution = "scal"
visoptions.subdivisions = 4

ngsolve.solve.Tcl_Eval("Ng_StandardRotation xz;\n")
